'use strict';

exports.__esModule = true;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = createInstance;

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _toConstant = require('../utils/toConstant');

var _toConstant2 = _interopRequireDefault(_toConstant);

var _makeActionHandler = require('./makeActionHandler');

var _makeActionHandler2 = _interopRequireDefault(_makeActionHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createInstanceByClass(ReducerClass, typePrefix) {
  var reducers = [];

  /* eslint-disable no-param-reassign */

  /**
   * Attaches reducer's handler function to specific action type
   * @param {Function} reducerHandler
   * @param {Object} actionCreator
   */
  ReducerClass.prototype.bindHandler = function bindHandler(reducerHandler, actionCreator) {
    reducers.push((0, _makeActionHandler2.default)(reducerHandler, typePrefix, actionCreator.toString(), this.initialState));
  };

  /**
   * Binds actionCreator for specific reducer handler function
   * @param {Object} actionCreator
   * @param {Function} reducerHandler
   */
  ReducerClass.prototype.bindAction = function bindAction(actionCreator, reducerHandler) {
    this.bindHandler(reducerHandler, actionCreator);
  };

  /* eslint-enable no-param-reassign */

  var reducer = new ReducerClass();

  return { reducers: reducers, initialState: reducer.initialState };
}

function createInstanceByObject(reducerObject, typePrefix) {
  var reducers = [];
  var prefixes = ['handle', 'reducerFor'];
  var initialState = reducerObject.initialState;

  var actionType = null;
  var reducerHandler = null;

  (0, _keys2.default)(reducerObject).forEach(function (reducer) {
    actionType = reducer;
    prefixes.forEach(function (prefix) {
      if (reducer.includes(prefix)) actionType = reducer.replace(prefix, '');
    });

    actionType = (0, _toConstant2.default)(actionType);
    reducerHandler = reducerObject[reducer];

    reducers.push((0, _makeActionHandler2.default)(reducerHandler, typePrefix, actionType, initialState));
  });

  return { reducers: reducers, initialState: initialState };
}

function createInstance(Reducer, typePrefix) {
  if ((0, _isFunction2.default)(Reducer)) return createInstanceByClass(Reducer, typePrefix);

  return createInstanceByObject(Reducer, typePrefix);
}